@@ -17,12 +17,13 @@ AUTHOR:        kingson fiasco
 #include "qtermbuffer.h"
 #include "qtermbbs.h"
 #include "qtermframe.h"
-#include "qtermwndmgr.h"
 #include "qtermparam.h"
 #include "qtermtelnet.h"
 #include "qtermconfig.h"
 #include "qtermglobal.h"
 #include "schemedialog.h"
+#include "osdmessage.h"
+#include "blur.h"
 
 #include <QApplication>
 #include <QPainter>
@@ -68,6 +69,7 @@ Screen::Screen(QWidget *parent, Buffer *buffer, Param *param, BBS *bbs)
     m_inputContent = NULL;
     m_pASCIIFont = NULL;
     m_pGeneralFont = NULL;
+    m_pMessage = new PageViewMessage(this);
 
     setFocusPolicy(Qt::ClickFocus);
     setAttribute(Qt::WA_InputMethodEnabled, true);
@@ -109,10 +111,15 @@ Screen::Screen(QWidget *parent, Buffer *buffer, Param *param, BBS *bbs)
 
 // init variable
     m_blinkScreen = false;
-    m_blinkCursor = true;
+    m_blinkCursor = m_pParam->m_mapParam["blinkcursor"].toBool();
 
 }
 
+PageViewMessage * Screen::osd()
+{
+    return m_pMessage;
+}
+
 Screen::~Screen()
 {
     delete [] m_pBlinkLine;
@@ -125,13 +132,15 @@ Screen::~Screen()
 // focus event received
 void Screen::focusInEvent(QFocusEvent *)
 {
-    if (m_pWindow->isMaximized() && m_pWindow->m_pFrame->wndmgr->afterRemove()) {
+    // FIXME: Somehow there is an inf loop without this.
+    if (Global::instance()->isFullScreen()) {
+        return;
+    }
+    if (m_pWindow->isMaximized()) {
         m_pWindow->showNormal();
         m_pWindow->showMaximized();
     }
 
-    m_pWindow->m_pFrame->wndmgr->activateTheTab(m_pWindow);
-
     m_scPrevPage->setEnabled(true);
     m_scNextPage->setEnabled(true);
     m_scPrevLine->setEnabled(true);
@@ -159,7 +168,23 @@ void Screen::cursorEvent()
     if (m_blinkCursor) {
         m_ePaintState = Cursor;
         m_bCursor = !m_bCursor;
-        update();
+        QRect cursorRect;
+        QPoint pt = mapToPixel(QPoint(m_pBuffer->caretX(), m_pBuffer->caretY()));
+        switch (m_pParam->m_mapParam["cursor"].toInt()) {
+        case 0:
+            cursorRect.setRect(pt.x(), pt.y(), m_nCharWidth, m_nCharHeight);
+            break;
+        case 1:
+            cursorRect.setRect(pt.x(), pt.y() + 9*m_nCharHeight / 10, m_nCharWidth, m_nCharHeight / 10);
+            break;
+        case 2:
+            cursorRect.setRect(pt.x(), pt.y(), m_nCharWidth / 9, m_nCharHeight);
+            break;
+        default:
+            qDebug("Unknown cursor type");
+            break;
+        }
+        update(cursorRect);
     }
 }
 
@@ -173,7 +198,7 @@ void Screen::updateCursor()
         int linelength = m_pBuffer->at(m_pBuffer->caretY())->getLength();
         int startx = m_pBuffer->caretX();
 
-        switch (m_pParam->m_nCursorType) {
+		switch (m_pParam->m_mapParam["cursor"].toInt()) {
         case 0: // block
             if (startx < linelength) {
                 drawLine(painter, m_pBuffer->caretY(), startx, startx, false);
@@ -208,14 +233,10 @@ void Screen::blinkEvent()
     if (m_hasBlink) {
         m_blinkScreen = !m_blinkScreen;
         m_ePaintState = Blink;
-        update();
+        update(m_blinkRegion);
     }
 }
 
-void Screen::moveEvent(QMoveEvent *)
-{
-// setBgPxm( m_pxmBg, m_nPxmType );
-}
 void Screen::resizeEvent(QResizeEvent *)
 {
     updateScrollBar();
@@ -227,7 +248,7 @@ void Screen::resizeEvent(QResizeEvent *)
 //  }
 //  m_pCanvas = new QPixmap(width(), height());
 
-    if (m_pParam->m_bAutoFont) {
+	if (m_pParam->m_mapParam["autofont"].toBool()) {
         updateFont();
     } else {
         int cx = m_rcClient.width() / m_nCharWidth;
@@ -238,56 +259,9 @@ void Screen::resizeEvent(QResizeEvent *)
     }
     m_ePaintState = Show;
     update();
+    QTimer::singleShot(50,this,SLOT(blurBackground()));
 }
 
-/* ------------------------------------------------------------------------ */
-/*                                                                         */
-/*                           Mouse                                          */
-/*                                                                          */
-/* ------------------------------------------------------------------------ */
-void Screen::enterEvent(QEvent * e)
-{
-    QApplication::sendEvent(m_pWindow, e);
-}
-
-void Screen::leaveEvent(QEvent * e)
-{
-    QApplication::sendEvent(m_pWindow, e);
-}
-
-void Screen::mousePressEvent(QMouseEvent * me)
-{
-    setFocus();
-
-    m_pWindow->mousePressEvent(me);
-    //QApplication::sendEvent(m_pWindow, me);
-
-}
-void Screen::mouseMoveEvent(QMouseEvent * me)
-{
-#ifdef Q_OS_MACX
-    m_pWindow->mouseMoveEvent(me);
-#else
-    m_pWindow->mouseMoveEvent(me);
-    //QApplication::sendEvent(m_pWindow, me);
-#endif
-}
-
-void Screen::mouseReleaseEvent(QMouseEvent * me)
-{
-    m_pWindow->mouseReleaseEvent(me);
-    //QApplication::sendEvent(m_pWindow, me);
-}
-
-void Screen::wheelEvent(QWheelEvent * we)
-{
-    if (Global::instance()->m_pref.bWheel)
-        QApplication::sendEvent(m_pWindow, we);
-    else
-        QApplication::sendEvent(m_scrollBar, we);
-}
-
-
 /* ------------------------------------------------------------------------ */
 /*                                                                         */
 /*                                  Font                                    */
@@ -302,33 +276,37 @@ void Screen::initFontMetrics()
     if (m_pGeneralFont != NULL) {
         delete m_pGeneralFont;
     }
-    m_pASCIIFont = new QFont(m_pParam->m_strASCIIFontName, qMax(8, m_pParam->m_nFontSize));
-    m_pGeneralFont = new QFont(m_pParam->m_strGeneralFontName, qMax(8, m_pParam->m_nFontSize));
+	m_pASCIIFont = new QFont(m_pParam->m_mapParam["asciifont"].toString(), 
+						qMax(8, m_pParam->m_mapParam["fontsize"].toInt()));
+    m_pGeneralFont = new QFont(m_pParam->m_mapParam["generalfont"].toString(), 
+						qMax(8, m_pParam->m_mapParam["fontsize"].toInt()));
 
     m_pASCIIFont->setWeight(QFont::Normal);
     m_pGeneralFont->setWeight(QFont::Normal);
 
     m_pASCIIFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
     m_pGeneralFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+    getFontMetrics();
+    m_pMessage->setFont(*m_pGeneralFont);
 }
 
 void Screen::updateFont()
 {
-    int nPixelSize;
+    int nPointSize;
     int nIniSize = qMax(8, qMin(m_rcClient.height() / m_pBuffer->line(),
                                 m_rcClient.width() * 2 / m_pBuffer->columns()));
 
-    for (nPixelSize = nIniSize - 3; nPixelSize <= nIniSize + 3; nPixelSize++) {
-        m_pASCIIFont->setPixelSize(nPixelSize);
-        m_pGeneralFont->setPixelSize(nPixelSize);
+    for (nPointSize = nIniSize - 3; nPointSize <= nIniSize + 3; nPointSize++) {
+        m_pASCIIFont->setPointSize(nPointSize);
+        m_pGeneralFont->setPointSize(nPointSize);
 
         getFontMetrics();
         if ((m_pBuffer->line()*m_nCharHeight) > m_rcClient.height()
                 || (m_pBuffer->columns()*m_nCharWidth) > m_rcClient.width()) {
-            while (nPixelSize > 5) {
-                nPixelSize--;
-                m_pASCIIFont->setPixelSize(nPixelSize);
-                m_pGeneralFont->setPixelSize(nPixelSize);
+            while (nPointSize > 5) {
+                nPointSize--;
+                m_pASCIIFont->setPointSize(nPointSize);
+                m_pGeneralFont->setPointSize(nPointSize);
 
                 getFontMetrics();
                 if ((m_pBuffer->line()*m_nCharHeight) < m_rcClient.height()
@@ -342,7 +320,6 @@ void Screen::updateFont()
     marginw=marginh=0;
     if (Frame::instance()->isMaximized()||Frame::instance()->isFullScreen()) {
         marginw = (m_rcClient.width() - (m_pBuffer->columns()*m_nCharWidth))/2;
-        marginh = (m_rcClient.height() - (m_pBuffer->line()*m_nCharHeight))/2;
     }
     QPoint point = m_rcClient.topLeft();
     m_rcClient = QRect(point.x()+marginw, point.y()+marginh, m_pBuffer->columns()*m_nCharWidth, m_pBuffer->line()*m_nCharHeight);
@@ -354,6 +331,7 @@ void Screen::updateFont()
         delete m_inputContent;
         m_inputContent = NULL;
     }
+    m_pMessage->setFont(*m_pGeneralFont);
 }
 
 void Screen::getFontMetrics()
@@ -368,7 +346,7 @@ void Screen::getFontMetrics()
         m_nCharWidth = (qMax(en, cn) + 1) / 2;
 
     m_nCharDelta = m_nCharWidth - cn/2;
-    m_nCharHeight = ascii_fm.height();
+    m_nCharHeight = qMax(ascii_fm.height(),general_fm.height());
     m_nCharAscent = ascii_fm.ascent();
     m_nCharDescent = ascii_fm.descent();
 }
@@ -380,8 +358,8 @@ void Screen::asciiFontChanged(const QFont & font)
         delete m_pASCIIFont;
     }
     m_pASCIIFont = new QFont(font);
-    m_pASCIIFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
-    m_pGeneralFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    m_pASCIIFont->setPointSize(qMax(8,m_pParam->m_mapParam["fontsize"].toInt()));
+    m_pGeneralFont->setPointSize(qMax(8,m_pParam->m_mapParam["fontsize"].toInt()));
     QResizeEvent* re = new QResizeEvent(size(), size());
     resizeEvent(re);
 }
@@ -392,17 +370,18 @@ void Screen::generalFontChanged(const QFont & font)
         delete m_pGeneralFont;
     }
     m_pGeneralFont = new QFont(font);
-    m_pASCIIFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
-    m_pGeneralFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    m_pASCIIFont->setPointSize(qMax(8,m_pParam->m_mapParam["fontsize"].toInt()));
+    m_pGeneralFont->setPointSize(qMax(8,m_pParam->m_mapParam["fontsize"].toInt()));
+    m_pMessage->setFont(*m_pGeneralFont);
     QResizeEvent* re = new QResizeEvent(size(), size());
     resizeEvent(re);
 }
 
 void Screen::fontSizeChanged(int value)
 {
-    m_pParam->m_nFontSize = value;
-    m_pASCIIFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
-    m_pGeneralFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    m_pParam->m_mapParam["fontsize"] = value;
+    m_pASCIIFont->setPointSize(qMax(8,value));
+    m_pGeneralFont->setPointSize(qMax(8,value));
     QResizeEvent* re = new QResizeEvent(size(), size());
     resizeEvent(re);
 }
@@ -419,7 +398,7 @@ QFont Screen::generalFont()
 
 int Screen::fontSize()
 {
-    return m_pParam->m_nFontSize;
+    return m_pParam->m_mapParam["fontsize"].toInt();
 }
 
 /* ------------------------------------------------------------------------ */
@@ -437,7 +416,7 @@ void Screen::setScheme()
     m_color[4]  = Qt::darkBlue;
     m_color[5]  = Qt::darkMagenta;
     m_color[6]  = Qt::darkCyan;
-    m_color[7]  = Qt::darkGray;
+    m_color[7]  = Qt::lightGray;
     m_color[8]  = Qt::gray;
     m_color[9]  = Qt::red;
     m_color[10] = Qt::green;
@@ -450,12 +429,12 @@ void Screen::setScheme()
     m_nPxmType = 0;
 
 
-
+	QString strSchemeFile = m_pParam->m_mapParam["schemefile"].toString();
 // if we have scheme defined
-    if (QFile::exists(m_pParam->m_strSchemeFile)) {
+    if (QFile::exists(strSchemeFile)) {
 
-//  printf("scheme %s loaded sucessfully\n", m_pParam->m_strSchemeFile);
-        Config *pConf = new Config(m_pParam->m_strSchemeFile);
+//  printf("scheme %s loaded sucessfully\n", strSchemeFile);
+        Config *pConf = new Config(strSchemeFile);
 
         m_color[0].setNamedColor(pConf->getItemValue("color", "color0").toString());
         m_color[1].setNamedColor(pConf->getItemValue("color", "color1").toString());
@@ -476,6 +455,7 @@ void Screen::setScheme()
 
         delete pConf;
     }
+    m_color[0].setAlphaF(m_pParam->m_mapParam["opacity"].toInt()/100.0);
 }
 
 void Screen::schemeChanged(int index)
@@ -484,12 +464,19 @@ void Screen::schemeChanged(int index)
         return;
     }
     QStringList schemeList = schemeDialog::loadSchemeList();
-    m_pParam->m_strSchemeFile = schemeList[index];
+    m_pParam->m_mapParam["schemefile"] = schemeList[index];
     setScheme();
     QResizeEvent* re = new QResizeEvent(size(), size());
     resizeEvent(re);
 }
 
+void Screen::opacityChanged(int val)
+{
+    m_color[0].setAlphaF(val/100.0);
+    QResizeEvent* re = new QResizeEvent(size(), size());
+    resizeEvent(re);
+}
+
 
 /* ------------------------------------------------------------------------ */
 /*                                                                         */
@@ -499,29 +486,24 @@ void Screen::schemeChanged(int index)
 void Screen::prevPage()
 {
     scrollLine(-m_pBuffer->line());
-    m_ePaintState = NewData;
-    update();
 }
 
 void Screen::nextPage()
 {
     scrollLine(m_pBuffer->line());
-    m_ePaintState = NewData;
-    update();
+
 }
 
 void Screen::prevLine()
 {
     scrollLine(-1);
-    m_ePaintState = NewData;
-    update();
+
 }
 
 void Screen::nextLine()
 {
     scrollLine(1);
-    m_ePaintState = NewData;
-    update();
+
 }
 
 void Screen::scrollLine(int delta)
@@ -546,6 +528,7 @@ void Screen::scrollLine(int delta)
     for (int i = m_nStart; i <= m_nEnd; i++)
         m_pBuffer->at(i)->setChanged(-1, -1);
 
+    updateRegion();
 }
 void Screen::scrollChanged(int value)
 {
@@ -565,9 +548,7 @@ void Screen::scrollChanged(int value)
     for (int i = m_nStart; i <= m_nEnd; i++)
         m_pBuffer->at(i)->setChanged(-1, -1);
 
-    m_ePaintState = NewData;
-    update();
-
+    updateRegion();
 }
 
 void Screen::updateScrollBar()
@@ -715,6 +696,7 @@ void Screen::blinkScreen()
                     startx = i;
                     while (i < linelength && GETBLINK(attr.at(i)))
                         ++i;
+                    painter.fillRect(mapToRect(startx, index, i-startx, 1), QBrush(m_color[0]));
                     --i;
                     drawLine(painter, index, startx, i, false);
                 }
@@ -740,6 +722,8 @@ void Screen::refreshScreen()
     int startx, endx;
 
     QPainter painter;
+    QRegion blinkRegion;
+
     painter.begin(this);
     //qDebug("size: %d, %d", width(),height());
     if (m_ePaintState == Show)
@@ -752,17 +736,28 @@ void Screen::refreshScreen()
             return;
         }
 
+        TextLine *pTextLine = m_pBuffer->at(index);
+        if (pTextLine->hasBlink()) {
+            m_hasBlink = true;
+            m_pBlinkLine[index - m_nStart] = true;
+            uint linelength = pTextLine->getLength();
+            QByteArray attr = pTextLine->getAttr();
+            for (uint i = 0; i < linelength; ++i) {
+                if (GETBLINK(attr.at(i))) {
+                    startx = i;
+                    while (i < linelength && GETBLINK(attr.at(i)))
+                        ++i;
+                    blinkRegion += mapToRect(startx, index, i-startx, 1);
+                }
+            }
+        } else
+            m_pBlinkLine[index - m_nStart] = false;
+
         if (m_ePaintState == Show) {
             drawLine(painter, index, 0, -1);
             continue;
         }
 
-        TextLine *pTextLine = m_pBuffer->at(index);
-        if (pTextLine->hasBlink()) {
-            m_hasBlink = true;
-            m_pBlinkLine[index - m_nStart] = true;
-        } else
-            m_pBlinkLine[index - m_nStart] = false;
         if (!pTextLine->isChanged(startx, endx))
             continue;
 
@@ -776,6 +771,7 @@ void Screen::refreshScreen()
         pTextLine->clearChange();
     }
     painter.end();
+    m_blinkRegion = blinkRegion;
 
     updateMicroFocus();
     if (m_pWindow->isConnected()) {
@@ -790,6 +786,11 @@ void Screen::refreshScreen()
     if (m_hasBlink) m_blinkTimer->start(1000);
 }
 
+void Screen::blurBackground()
+{
+    if (m_pParam->m_mapParam["opacity"].toInt() < 100)
+        BlurHelper().updateBlurRegion(Frame::instance(), Frame::instance()->rect());
+}
 
 void Screen::paintEvent(QPaintEvent * pe)
 {
@@ -855,7 +856,7 @@ void Screen::repaintScreen(QPaintEvent * pe)
 
     for (int y = tlPoint.y(); y <= brPoint.y(); y++) {
         drawLine(painter, y);
-        //if( m_pBBS->isSelected(y)&&m_pParam->m_nMenuType==1 )
+        //if( m_pBBS->isSelected(y)&&m_pParam->m_mapParam["menutype"].toInt()==1 )
         //{
         // QRect rcMenu = mapToRect(m_pBBS->getSelectRect());
         // QPixmap pxm(rcMenu.width(), rcMenu.height());
@@ -889,12 +890,21 @@ void Screen::drawLine(QPainter& painter, int index, int beginx, int endx, bool c
     //qDebug() << "beginx: " << beginx << ", endx: " << endx << ", linelength: " << linelength;
     QString strShow;
 
+	if (complete == true && m_pBBS->isSelected(index)) {
+		drawMenuSelect(painter, index);
+		if (m_pParam->m_mapParam["menutype"].toInt() == 1) {
+			bReverse = true;
+		}
+		beginx = 0;
+		endx = linelength - 1;
+	}
+
     if (beginx < 0)
         beginx = 0;
 
     if (endx >= linelength || endx < 0) {
         endx = qMin(m_pBuffer->columns(), linelength)-1;
-        painter.eraseRect(mapToRect(beginx, index, linelength, 1)); // Maybe we should calculate more accurate size;
+        //painter.eraseRect(mapToRect(beginx, index, linelength, 1)); // Maybe we should calculate more accurate size;
     }
     if (endx >= qMin(color.size(), attr.size())) {
         endx = qMin(color.size(), attr.size()) -1;
@@ -905,15 +915,6 @@ void Screen::drawLine(QPainter& painter, int index, int beginx, int endx, bool c
         return;
     }
 
-    if (complete == true && m_pBBS->isSelected(index)) {
-        drawMenuSelect(painter, index);
-        if (m_pParam -> m_nMenuType == 1) {
-            bReverse = true;
-        }
-        beginx = 0;
-        endx = linelength -1;
-    }
-
     if (beginx > 0 && pTextLine->isPartial(beginx)) {
         beginx -= 1;
     }
@@ -928,7 +929,7 @@ void Screen::drawLine(QPainter& painter, int index, int beginx, int endx, bool c
             tempcp = color.at(i);
         if (i < attr.size())
             tempea = attr.at(i);
-        bSelected = m_pBuffer->isSelected(QPoint(i, index), m_pWindow->m_bCopyRect);
+        bSelected = m_pBuffer->isSelected(QPoint(i, index), m_pWindow->m_bRectCopy);
         len = pTextLine->size(i);
         if ( (i+1) >= linelength) {
             len = 1;
@@ -948,28 +949,24 @@ void Screen::drawLine(QPainter& painter, int index, int beginx, int endx, bool c
         //qDebug() << "startx: " << startx << " i: " << i << " string: " << strShow;
         // There should be only one.
         // TODO: Rewrite this when we want to do more than char to char convert
-        strShow = Global::instance()->convert(pTextLine->getText(startx, len), (Global::Conversion)m_pParam->m_nDispCode);
+        strShow = Global::instance()->convert(pTextLine->getText(startx, len), (Global::Conversion)m_pParam->m_mapParam["displaycode"].toInt());
 
         if (strShow.isEmpty()) {
             qDebug("drawLine: empty string?");
-            continue;
+            break;
         }
         // Draw Charactors one by one to fix the variable font display problem
         int charWidth = TermString::wcwidth(strShow.at(0));
         if (charWidth <= 0) {
             qDebug("drawLine: non printable char");
             continue;
-        } else if (charWidth == 1) {
-            painter.setFont(*m_pASCIIFont);
-        } else if (charWidth == 2) {
-            painter.setFont(*m_pGeneralFont);
         }
         CharFlags flags = RenderAll;
         if ( pTextLine->isPartial(startx) ) {
             flags = RenderRight;
             charWidth = 1;
         } else if ( charWidth == 2) {
-            if (tempcp != color.at(i+1) || tempea != attr.at(i+1) || bSelected != m_pBuffer->isSelected(QPoint(i+1, index), m_pWindow->m_bCopyRect)) {
+            if (tempcp != color.at(i+1) || tempea != attr.at(i+1) || bSelected != m_pBuffer->isSelected(QPoint(i+1, index), m_pWindow->m_bRectCopy)) {
                 charWidth = 1;
                 flags = RenderLeft;
             } else {
@@ -990,18 +987,22 @@ void Screen::drawStr(QPainter& painter, const QString& str, int x, int y, int le
     char ea = GETATTR(attribute);
 
     // test bold mask or always highlighted
-    if (GETBOLD(ea) || m_pParam->m_bAlwaysHighlight)
+    if (GETBOLD(ea) || m_pParam->m_mapParam["alwayshighlight"].toBool())
         cp = SETHIGHLIGHT(cp);             // use 8-15 color
     // test dim mask
     if (GETDIM(ea)) {
     };
     // test underline mask
+    QFont asciiFont = *m_pASCIIFont;
+    QFont generalFont = *m_pGeneralFont;
     if (GETUNDERLINE(ea)) {
-        m_pASCIIFont->setUnderline(true);
-        m_pGeneralFont->setUnderline(true);
-    } else {
-        m_pASCIIFont->setUnderline(false);
-        m_pGeneralFont->setUnderline(false);
+        asciiFont.setUnderline(true);
+        generalFont.setUnderline(true);
+    }
+    if (length == 1) {
+       painter.setFont(asciiFont);
+    } else if (length == 2) {
+       painter.setFont(generalFont);
     }
     // test blink mask
     if (GETBLINK(ea)) {
@@ -1031,31 +1032,38 @@ void Screen::drawStr(QPainter& painter, const QString& str, int x, int y, int le
         return;
     }
 
-    painter.setPen(m_color[m_pParam->m_bAnsiColor||GETFG(cp)==0?GETFG(cp):7]);
+    painter.setPen(m_color[m_pParam->m_mapParam["ansicolor"].toBool()||GETFG(cp)==0?GETFG(cp):7]);
 
     if (GETBG(cp) != 0 && !transparent) {
         painter.setBackgroundMode(Qt::OpaqueMode);
-        painter.setBackground(m_color[m_pParam->m_bAnsiColor||GETBG(cp)==7?GETBG(cp):0]);
+        painter.setBackground(m_color[m_pParam->m_mapParam["ansicolor"].toBool()||GETBG(cp)==7?GETBG(cp):0]);
 
     } else
         painter.setBackgroundMode(Qt::TransparentMode);
 
-
     if (m_blinkScreen && GETBLINK(GETATTR(attribute))) {
         if (GETBG(cp) != 0)
             painter.fillRect(mapToRect(x, y, length, 1), QBrush(m_color[GETBG(cp)]));
-        else
-            painter.eraseRect(mapToRect(x, y, length, 1));
     } else {
         if (GETBG(cp) != 0 || m_ePaintState == Cursor)
             painter.fillRect(mapToRect(x, y, length, 1), QBrush(m_color[GETBG(cp)]));
         if (flags == RenderAll) {
-            painter.drawText(pt.x()+m_nCharDelta, pt.y(), m_nCharWidth*length, m_nCharHeight, Qt::AlignLeft|Qt::AlignBottom, str);
-        } else if (flags == RenderLeft) {
-            painter.drawText(pt.x()+m_nCharDelta, pt.y(), m_nCharWidth-m_nCharDelta, m_nCharHeight, Qt::AlignLeft|Qt::AlignBottom, str);
-        } else if (flags == RenderRight) {
-            int width = painter.fontMetrics().width(str[0])-m_nCharWidth+m_nCharDelta;
-            painter.drawText(pt.x(), pt.y(), width, m_nCharHeight, Qt::AlignRight|Qt::AlignBottom, str);
+            //painter.fillRect(mapToRect(x, y, length, 1), QBrush(m_color[0]));
+            painter.drawText(pt.x(), pt.y(), m_nCharWidth*length, m_nCharHeight, Qt::AlignCenter, str);
+        } else {
+            QPixmap pm = QPixmap(m_nCharWidth*2, m_nCharHeight);
+            QPainter p(&pm);
+            p.fillRect(0, 0, pm.width(), pm.height(), m_color[GETBG(cp)]);
+            p.setPen(m_color[GETFG(cp)]);
+            p.setFont(painter.font());
+            p.drawText(0, 0, m_nCharWidth*2, m_nCharHeight, Qt::AlignCenter, str);
+            p.end();
+            if (flags == RenderLeft) {
+                painter.drawPixmap(pt.x(), pt.y(), pm, 0, 0, m_nCharWidth, m_nCharHeight);
+            }
+            if (flags == RenderRight) {
+                painter.drawPixmap(pt.x(), pt.y(), pm, m_nCharWidth, 0, m_nCharWidth, m_nCharHeight);
+            }
         }
     }
     painter.setBackground(QBrush(m_color[0]));
@@ -1079,7 +1087,7 @@ void Screen::drawMenuSelect(QPainter& painter, int index)
 {
     QRect rcSelect, rcMenu, rcInter;
     if (m_pBuffer->isSelected(index)) {
-        rcSelect = mapToRect(m_pBuffer->getSelectRect(index, m_pWindow->m_bCopyRect));
+        rcSelect = mapToRect(m_pBuffer->getSelectRect(index, m_pWindow->m_bRectCopy));
         if (Global::instance()->isBossColor())
             painter.fillRect(rcSelect, Qt::black);
         else
@@ -1088,17 +1096,36 @@ void Screen::drawMenuSelect(QPainter& painter, int index)
 
     if (m_pBBS->isSelected(index)) {
         rcMenu = mapToRect(m_pBBS->getSelectRect().intersected(QRect(0,index,m_pBuffer->columns(), 1)));
-        switch (m_pParam->m_nMenuType) {
+        switch (m_pParam->m_mapParam["menutype"].toInt()) {
         case 0: // underline
             painter.fillRect(rcMenu.x(), rcMenu.y() + 10*m_nCharHeight / 11, rcMenu.width(), m_nCharHeight / 11, m_color[7]);
             break;
         case 2:
-            painter.fillRect(rcMenu, QBrush(m_pParam->m_clrMenu));
+            painter.fillRect(rcMenu, QBrush(m_pParam->m_mapParam["menucolor"].value<QColor>()));
             break;
         }
     }
 }
 
+void Screen::updateRegion()
+{
+    int startx, endx;
+
+    m_ePaintState = NewData;
+    QRegion paintRegion;
+    for (int index = m_nStart; index <= m_nEnd; index++) {
+        TextLine *pTextLine = m_pBuffer->at(index);
+        if (!pTextLine->isChanged(startx, endx))
+            continue;
+        if (startx > 0 && pTextLine->isPartial(startx)) {
+            startx -= 1;
+        }
+
+        paintRegion += mapToRect(startx, index, -1, 1);
+    }
+    update(paintRegion);
+}
+
 /* ------------------------------------------------------------------------ */
 /*                                                                         */
 /*                              Auxilluary                                 */
@@ -1133,12 +1160,15 @@ QPoint Screen::mapToChar(const QPoint& point)
 
 QRect Screen::mapToRect(int x, int y, int width, int height)
 {
-    QPoint pt = mapToPixel(QPoint(x, y));
+    int px = x < 0 ? 0 : x;
+    int py = y < 0 ? 0 : y;
+
+    QPoint pt = mapToPixel(QPoint(px, py));
 
     if (width == -1)  // to the end
-        return QRect(pt.x()+m_nCharDelta, pt.y(), size().width() , m_nCharHeight*height);
+        return QRect(pt.x(), pt.y(), size().width() , m_nCharHeight*height);
     else
-        return QRect(pt.x()+m_nCharDelta, pt.y(), width*m_nCharWidth, m_nCharHeight*height);
+        return QRect(pt.x(), pt.y(), width*m_nCharWidth, m_nCharHeight*height);
 }
 
 QRect Screen::mapToRect(const QRect& rect)
@@ -1169,7 +1199,7 @@ QImage& Screen::fade(QImage& img, float val, const QColor& color)
 
     if (img.depth() <= 8) {
         // pseudo color
-        for (int i = 0; i < img.numColors(); i++) {
+        for (int i = 0; i < img.colorCount(); i++) {
             col = img.color(i);
             cr = qRed(col); cg = qGreen(col); cb = qBlue(col);
             if (cr > red)
@@ -1331,4 +1361,4 @@ void Input::paintEvent(QPaintEvent * e)
 
 } // namespace QTerm
 
-#include <qtermscreen.moc>
+#include <moc_qtermscreen.cpp>
