@@ -78,12 +78,9 @@ struct fsm_trans Telnet::substab[] = {
  *------------------------------------------------------------------------
  */
 Telnet::Telnet(const QString & strTermType, int rows, int columns, bool isSSH)
-        : from_socket(), to_ansi(), from_ansi(), to_socket()
+        : term(), from_socket(), to_ansi(), from_ansi(), to_socket()
 {
-    term = new char[21];
-    int i;
-    for (i = 0;i < 21;i++) term[i] = '\000'; //clean up, need???
-    sprintf(term, strTermType.toLatin1());
+    term = strTermType.toUtf8();
 
     wx = columns;
     wy = rows;
@@ -99,6 +96,8 @@ Telnet::Telnet(const QString & strTermType, int rows, int columns, bool isSSH)
 
     raw_size = 0;
 
+    bConnected = false;
+
     // create socket
     d_isSSH = isSSH;
 #ifdef SSH_ENABLED
@@ -117,8 +116,6 @@ Telnet::Telnet(const QString & strTermType, int rows, int columns, bool isSSH)
             this, SLOT(showError(QAbstractSocket::SocketError)));
     connect(socket, SIGNAL(hostFound()),
             this, SLOT(hostFound()));
-    connect(socket, SIGNAL(delayedCloseFinished()),
-            this, SLOT(delayCloseFinished()));
     connect(socket, SIGNAL(connectionClosed()),
             this, SLOT(closed()));
     connect(socket, SIGNAL(SocketState(int)),
@@ -134,7 +131,6 @@ Telnet::Telnet(const QString & strTermType, int rows, int columns, bool isSSH)
 Telnet::~Telnet()
 {
     // delete objects
-    delete [] term;
     delete socket;
 }
 
@@ -218,7 +214,15 @@ void Telnet::windowSizeChanged(int x, int y)
 {
     wx = x;
     wy = y;
-    if (bConnected) {
+    done_naws = 0;
+    if (!bConnected)
+        return;
+
+    if (d_isSSH) {
+        SSHSocket *sshSocket = qobject_cast<SSHSocket*>(socket);
+        if (sshSocket)
+                sshSocket->requestWindowSize(x, y);
+    } else {
         naws = 0;
 
         QByteArray cmd(10, 0);
@@ -234,6 +238,7 @@ void Telnet::windowSizeChanged(int x, int y)
         socket->writeBlock(cmd);
 
     }
+    done_naws = 1;
 }
 
 /*------------------------------------------------------------------------
@@ -267,6 +272,9 @@ void Telnet::close()
 void Telnet::connected()
 {
     bConnected = true;
+    // in case the window size changed before connection was made
+    if (done_naws == 0)
+        windowSizeChanged(wx, wy);
     emit TelnetState(TSHOSTCONNECTED);
 }
 /*------------------------------------------------------------------------
@@ -281,16 +289,6 @@ void Telnet::closed()
 
     emit TelnetState(TSCLOSED);
 }
-/*------------------------------------------------------------------------
- * SLOT delayCloseFinished
- *------------------------------------------------------------------------
- */
-
-void Telnet::delayCloseFinished()
-{
-    bConnected = false;
-    emit TelnetState(TSCLOSEFINISH);
-}
 /*------------------------------------------------------------------------
  * SLOT hostFound
  *------------------------------------------------------------------------
@@ -564,7 +562,7 @@ int Telnet::do_echo(int c)
 void Telnet::putc_down(u_char c)
 {
     // check overflow
-    if ((wsize + 1) > to_socket.size()) {
+    if ((wsize + 1) > (uint) to_socket.size()) {
         qWarning("putc_down : to_socket buffer overflow");
         return;
     }
@@ -767,7 +765,6 @@ int Telnet::subopt(int c)
 
 int Telnet::subtermtype(int)
 {
-    char *i;
     /* have received IAC.SB.TERMTYPE.SEND */
 
     putc_down(TCIAC);
@@ -777,8 +774,8 @@ int Telnet::subtermtype(int)
 
     //write term type string
     //fputs(term, sfp);
-    for (i = term; (*i) != '\000'; i++)
-        putc_down(*i);
+    for (int i = 0; i < term.length(); i++)
+        putc_down(term[i]);
 
     putc_down(TCIAC);
     putc_down(TCSE);
@@ -846,7 +843,7 @@ int Telnet::xputc_up(char ch)
     /*if (scrfp)
      (void) putc(ch, scrfp);*/
 
-    if ((rsize + 1) > to_ansi.size()) {
+    if ((rsize + 1) > (uint) to_ansi.size()) {
         qWarning("xputc_up : Buffer to_ansi overflow");
         return -1;
     }
@@ -908,4 +905,4 @@ int Telnet::tnabort(int)
 
 } // namespace QTerm
 
-#include <qtermtelnet.moc>
+#include <moc_qtermtelnet.cpp>
