@@ -135,7 +135,6 @@ private slots:
     void socketReadyRead();
     void showError(QAbstractSocket::SocketError);
     void hostFound();
-    void delayCloseFinished();
     void closed();
 protected:
     //init structure fsm
@@ -216,7 +215,7 @@ private:
     int done_naws;
     bool d_isSSH;
     bool bConnected;
-    int raw_size;
+    uint raw_size;
 };
 
 } // namespace QTerm
