@@ -19,8 +19,8 @@ AUTHOR:        kingson fiasco
 
 namespace QTerm
 {
-TextLine::TextLine()
-        : m_text(), m_color(), m_attr()
+TextLine::TextLine(QObject * parent)
+        : QObject(parent), m_text(), m_color(), m_attr()
 {
     m_bChanged = true;
     m_start = -1;
@@ -73,19 +73,19 @@ void TextLine::insertText(const QString & str, short attribute, int index)
             tmp.fill(NO_COLOR, index - m_length);
             m_color.append(tmp);
             tmp.fill(m_curColor, len);
-            m_color.insert(index, tmp.data());
+            m_color.insert(index, tmp);
 
             tmp.fill(NO_ATTR, index - m_length);
             m_attr.append(tmp);
             tmp.fill(m_curAttr, len);
-            m_attr.insert(index, tmp.data());
+            m_attr.insert(index, tmp);
         } else {
             m_text.insert(index, str);
             m_length = m_text.length();
             tmp.fill(m_curColor, len);
-            m_color.insert(index, tmp.data());
+            m_color.insert(index, tmp);
             tmp.fill(m_curAttr, len);
-            m_attr.insert(index, tmp.data());
+            m_attr.insert(index, tmp);
 
         }
         start = index;
@@ -231,7 +231,7 @@ QString TextLine::getAttrText(int index, int len, const QString & escape)
 
 
     if (index >= m_length)
-        return (char *)NULL;
+        return QString();
 
     //qDebug("index=%d len=%d m_length=%d", index, len, m_length);
 
@@ -248,9 +248,7 @@ QString TextLine::getAttrText(int index, int len, const QString & escape)
 
         int fg = GETFG(tempcp) + 30;
         int bg = GETBG(tempcp) + 40;
-        QString strAttr = QString("%1;%2").arg(fg).arg(bg).toLatin1();//QByteArray::setNum(fg)+';'+QByteArray::setNum(bg)+';';
-//   cstrAttr.sprintf("%d;%d;", fg, bg );
-        strAttr = escape + strAttr;
+        QString strAttr = escape;
 
         if (GETBOLD(tempea))
             strAttr += "1;";
@@ -266,17 +264,15 @@ QString TextLine::getAttrText(int index, int len, const QString & escape)
             strAttr += "7;";
         if (GETINVISIBLE(tempea))
             strAttr += "8;";
-        strAttr.remove(strAttr.length() - 1, 1);
-        strAttr += "m";
+        strAttr += QString("%1;%2m").arg(fg).arg(bg);
         str += strAttr; // set attr
+
         // the text
         str += getText(startx, i - startx);
-        // reset attr
-        strAttr = escape + "0m";
-        str += strAttr;
         i--;
     }
 
+    str += escape + "m";
     return str;
 }
 
@@ -373,4 +369,4 @@ void TextLine::setAttr(short attr, int index)
 }
 
 } // namespace QTerm
-#include <qtermtextline.moc>
+#include <moc_qtermtextline.cpp>
