@@ -5,6 +5,7 @@
 #include <config.h>
 #endif
 
+#include "qtermwindowbase.h"
 #include "qtermparam.h"
 #include "qtermconvert.h"
 #include "qtermsound.h"
@@ -52,6 +53,7 @@ class PageViewMessage;
 class ScriptHelper;
 #endif
 class HostInfo;
+class KeyboardTranslator;
 // thread copy article
 class DAThread : public QThread
 {
@@ -69,12 +71,12 @@ private:
     Window *pWin;
 };
 
-class Window: public QMainWindow
+class Window: public WindowBase
 {
     Q_OBJECT
 public:
-    Window(Frame * frame, Param param, int addr = -1, QWidget* parent = 0,
-           const char* name = 0, Qt::WFlags wflags = Qt::Window);
+    Window(Frame * frame, Param param, const QString& uuid, QWidget* parent = 0,
+           const char* name = 0, Qt::WindowFlags wflags = Qt::Window);
     ~Window();
 
     void connectHost();
@@ -82,33 +84,59 @@ public:
         return m_bConnected;
     }
 
+    QString uuid() {
+        return m_strUuid;
+    }
+
+
+
+public slots:
+	// File
+	void on_actionDisconnect_triggered();
+	// Edit
+    void on_actionCopy_triggered();
+	void on_actionPaste_triggered();
+	void on_actionRectangle_Selection_toggled(bool rect) { m_bRectCopy = rect; }
+	void on_actionCopy_w_Color_toggled(bool color) { m_bColorCopy = color; }
+	void on_actionAuto_Copy_toggled(bool automatic) { m_bAutoCopy = automatic; }
+	void on_actionPaste_w_Wrodwrap(bool wordwrap) { m_bWordWrap = wordwrap; }
+	void on_actionUnderline_toggled(bool underline);
+	void on_actionBlink_toggled(bool blink);
+	void on_actionPallete_triggered(const QVariant& data);
+	void on_actionSymbols_triggered(const QVariant& data);
+	// View
+	void on_actionRefresh_triggered();
+	void on_actionBoss_Color_toggled(bool);
+	// BBS
+	void on_actionAuto_Reply_toggled(bool);
+	void on_actionAnti_Idle_toggled(bool);
+	void on_actionCopy_Article_triggered();
+	void on_actionView_Message_triggered();
+	void on_actionBeep_on_message_toggled(bool beep) { m_bBeep = beep; }
+	void on_actionSupport_Mouse_toggled(bool mouse) { m_bMouse = mouse; }
+
+	// Option
+	void on_actionCurrent_Session_Setting_triggered();
+	// Script
+	void on_actionRun_triggered();
+	void on_actionStop_triggered();
+	void on_actionDebug_Console_triggered();
+	void on_actionReload_Script_triggered();
+
 public slots:
     // ui
-    void copy();
-    void paste();
-    void copyArticle();
-    void setting();
-    void appearance();
-    void runScript(const QString & filename="");
-    void stopScript();
-    void debugConsole();
-    void showStatusBar(bool);
     void reconnect();
     void sendParsedString(const QString &);
     void showIP();
     void inputHandle(const QString & text);
 public:
-    void disconnect();
-    void refresh();
-    void viewMessages();
-    void autoReply(bool);
-    void antiIdle(bool);
-
     void initScript();
+	void runScript(const QString & filename="");
     void externInput(const QString &);
     void getHttpHelper(const QString&, bool);
-    void showMessage(const QString &, int type, int duration);
-    Screen * getScreen()
+    void osdMessage(const QString &, int type, int duration);
+    void showMessage(const QString & title, const QString & message, int duration = -1);
+    Screen * screen()
     {
         return m_pScreen;
     }
@@ -132,7 +160,6 @@ protected slots:
     void idleProcess();
     void replyProcess();
     void updateWindow();
-    void blinkTab();
 
     //http menu
     void previewLink();
@@ -144,10 +171,13 @@ protected slots:
     void httpDone(QObject*);
 
     // decode
-    void setMouseMode(bool);
+	void setMouseMode(bool on ) { m_bMouseX11 = on; }
     void jobDone(int);
+    void showArticle(const QString text);
+    void updateHostKey(const QString & hostKey);
 
 protected:
+	// Mouse Events
     void mouseDoubleClickEvent(QMouseEvent *);
     void mouseMoveEvent(QMouseEvent *);
     void mousePressEvent(QMouseEvent *);
@@ -156,16 +186,9 @@ protected:
     void enterEvent(QEvent *);
     void leaveEvent(QEvent *);
 
-//   void customEvent( QCustomEvent * );
-    /*
-    void imStartEvent (QIMEvent *);
-    void imComposeEvent (QIMEvent *);
-    void imEndEvent (QIMEvent *);
-    */
     void reconnectProcess();
     void connectionClosed();
     void doAutoLogin();
-    void saveSetting();
     void replyMessage();
 
     void pasteHelper(bool);
@@ -176,12 +199,16 @@ protected:
     void closeEvent(QCloseEvent *);
     void keyPressEvent(QKeyEvent *);
 
+    void loadKeyboardTranslator(const QString & profile);
+	
+	void groupActions();
+
     Screen * m_pScreen;
     Decode * m_pDecode;
     QMenu *  m_pMenu;
     QMenu *  m_pUrl;
     static char direction[][5];
-    QCursor cursor[9];
+    QCursor cursor[10];
     bool m_bConnected;
 
     Convert m_converter;
@@ -196,12 +223,11 @@ protected:
 
     // timer
     bool m_bIdling;
-    QTimer *m_idleTimer, *m_replyTimer, *m_tabTimer, *m_reconnectTimer, * m_ipTimer;
+    QTimer *m_idleTimer, *m_replyTimer, *m_reconnectTimer, * m_ipTimer;
     QTimer * m_updateTimer;
 
     // address setting
-    bool m_bSetChanged;
-    int m_nAddrIndex;
+    QString m_strUuid;
 
     bool m_bDoingLogin;
     bool m_bCheckIP;
@@ -240,6 +266,7 @@ protected:
     bool m_bMouseClicked;
 
     QTextCodec * m_codec;
+    KeyboardTranslator * m_translator;
 #ifdef SCRIPT_ENABLED
     QScriptEngine * m_scriptEngine;
     ScriptHelper * m_scriptHelper;
@@ -255,8 +282,8 @@ public:
     BBS * m_pBBS;
     HostInfo * m_hostInfo;
     // menu and toolbar state
-    bool m_bCopyColor;
-    bool m_bCopyRect;
+    bool m_bColorCopy;
+    bool m_bRectCopy;
     bool m_bAutoCopy;
     bool m_bWordWrap;
     bool m_bAntiIdle;
