@@ -1,10 +1,14 @@
 #include "toolbardialog.h"
+#include "qtermglobal.h"
 
-#include <QtGui/QToolBar>
 #include <QtCore/QRegExp>
 #include <QtCore/QSettings>
-#include <QtGui/QMainWindow>
-#include <QtGui/QToolButton>
+
+#include <QToolBar>
+#include <QMainWindow>
+#include <QToolButton>
+
+namespace QTerm {
 
 ToolbarDialog::ToolbarDialog(QWidget* parent)
         : QDialog(parent),m_defaultToolBars()
@@ -16,7 +20,7 @@ ToolbarDialog::ToolbarDialog(QWidget* parent)
     QList<QAction*> actions = parent->findChildren<QAction*>(QRegExp("action*"));
     QAction* action;
     foreach(action, actions) {
-        if (action->actionGroup() != 0)
+        if (action->actionGroup()->objectName() != "extraGroup")
             continue;
         QListWidgetItem* item = new QListWidgetItem(action->toolTip());
         item->setIcon(action->icon());
@@ -30,8 +34,8 @@ ToolbarDialog::ToolbarDialog(QWidget* parent)
     QToolBar* toolbar = NULL;
     int index = 0;
     foreach(toolbar, toolbars) {
-        index = toolbar->iconSize().height()/16-1;
-        if (toolbar->objectName() != "customKeyToolBar")
+        index = (int)(toolbar->iconSize().height()/10)-1;
+        if (toolbar->objectName() != "keyToolBar")
             comboToolbars->addItem(toolbar->windowTitle(), QVariant::fromValue((QObject*)toolbar));
     }
     comboToolbarsCurrentIndexChanged(0);
@@ -48,10 +52,12 @@ ToolbarDialog::ToolbarDialog(QWidget* parent)
     connect(comboIconSize, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(comboIconSizeCurrentIndexChanged(const QString &)));
     connect(comboButtonStyle, SIGNAL(currentIndexChanged(int)),this, SLOT(comboButtonStyleCurrentIndexChanged(int)));
     connect(buttonDefault, SIGNAL(clicked()), this, SLOT(restoreDefaultToolbars()));
+    restoreGeometry(Global::instance()->loadGeometry("Toolbar"));
 }
 
 ToolbarDialog::~ToolbarDialog()
 {
+    Global::instance()->saveGeometry("Toolbar",saveGeometry());
 }
 
 void ToolbarDialog::buttonAddClicked()
@@ -177,6 +183,8 @@ void ToolbarDialog::comboIconSizeCurrentIndexChanged(const QString& item)
     QSize iconSize;
     if (item == "16x16")
         iconSize = QSize(16,16);
+    else if (item == "22x22")
+        iconSize = QSize(22,22);
     else if (item == "32x32")
         iconSize = QSize(32,32);
     else if (item == "48x48")
@@ -214,11 +222,11 @@ void ToolbarDialog::createDefaultToolBars()
 {
     QStringList listActions;
     listActions << "actionQuickConnect";
-    m_defaultToolBars.insert("mainToolBar", listActions);
+    m_defaultToolBars.insert("terminalToolBar", listActions);
 
     listActions.clear();
     listActions << "actionDisconnect" << "Separator" << "actionCopy" << "actionPaste" << "actionRect" << "actionColorCopy" << "Separator" << "actionAppearance" << "actionRefresh" << "Separator" << "actionCurrentSession" << "Separator" << "actionCopyArticle" << "actionAntiIdle" << "actionAutoReply" << "actionViewMessage" << "actionMouse" << "actionBeep" << "actionReconnect";
-    m_defaultToolBars.insert("bbsOperationsToolBar", listActions);
+    m_defaultToolBars.insert("mainToolBar", listActions);
 }
 
 void ToolbarDialog::restoreDefaultToolbars()
@@ -246,4 +254,6 @@ void ToolbarDialog::restoreDefaultToolbars()
     comboToolbarsCurrentIndexChanged(comboToolbars->currentIndex());
 }
 
-#include <toolbardialog.moc>
+} // namespace QTerm
+
+#include <moc_toolbardialog.cpp>
